/*
 * IsLanguageGeneratingEpsilon.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include "IsLanguageGeneratingEpsilon.h"
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace properties {

auto IsLanguageGeneratingEpsilonCFG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::CFG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonEpsilonFreeCFG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::EpsilonFreeCFG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonGNF = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::GNF < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonCNF = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::CNF < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonLG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::LG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonLeftLG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::LeftLG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonLeftRG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::LeftRG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonRightLG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::RightLG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

auto IsLanguageGeneratingEpsilonRightRG = registration::AbstractRegister < IsLanguageGeneratingEpsilon, bool, const grammar::RightRG < > & > ( IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon, "grammar" ).setDocumentation (
"Decides whether \\e in L( grammar )\n\
Severals steps implemented in method @see grammar::properties::NullableNonterminals::getNullableNonterminals();\n\
\n\
@param grammar the tested grammar\n\
@returns true if \\e in L(@p grammar)" );

} /* namespace properties */

} /* namespace grammar */

