/*
 * LeftRecursionRemover.cpp
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#include "LeftRecursionRemover.h"
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace simplify {

auto LeftRecursionRemoverEpsilonFreeCFG = registration::AbstractRegister < LeftRecursionRemover, grammar::EpsilonFreeCFG < >, const grammar::EpsilonFreeCFG < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

auto LeftRecursionRemoverCNF = registration::AbstractRegister < LeftRecursionRemover, grammar::EpsilonFreeCFG < >, const grammar::CNF < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

auto LeftRecursionRemoverGNF = registration::AbstractRegister < LeftRecursionRemover, grammar::GNF < >, const grammar::GNF < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

auto LeftRecursionRemoverRightRG = registration::AbstractRegister < LeftRecursionRemover, grammar::RightRG < >, const grammar::RightRG < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

auto LeftRecursionRemoverRightLG = registration::AbstractRegister < LeftRecursionRemover, grammar::RightLG < >, const grammar::RightLG < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

auto LeftRecursionRemoverLeftRG = registration::AbstractRegister < LeftRecursionRemover, grammar::RightRG < >, const grammar::LeftRG < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

auto LeftRecursionRemoverLeftLG = registration::AbstractRegister < LeftRecursionRemover, grammar::RightLG < >, const grammar::LeftLG < > & > ( LeftRecursionRemover::remove, "grammar" ).setDocumentation (
"Removes left recursion from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without left recursion" );

} /* namespace simplify */

} /* namespace grammar */
