/*
 * Dot.cpp
 *
 *  Created on: 16. 8. 2017
 *	  Author: Jan Travnicek
 */

#include "Dot.h"
#include <cstdlib>
#include <cctype>

#include <alib/algorithm>
#include <numeric>
#include <registration/AlgoRegistration.hpp>
#include <exception/CommonException.h>

#include <unistd.h>


namespace cli {

namespace builtin {

std::vector < std::string> Dot::allowedTypes { "dot", "xdot", "ps", "pdf", "svg", "svgz", "fig", "png", "gif", "jpg", "jpeg", "json", "imap", "cmapx" };

void Dot::run ( const std::string & data, const std::string & outputType, const std::string & outputFile ) {
	int fd [ 2 ];
	if ( pipe ( fd ) != 0 )
		throw exception::CommonException ( "Dot: Pipe failure." );

	pid_t pid = fork ( );

	if ( pid < 0 ) {
		throw exception::CommonException ( "Dot: Fork failure." );
	} else if ( pid == 0 ) {
		close ( STDIN_FILENO );
		close ( STDOUT_FILENO );
		dup2 ( fd [ 0 ], STDIN_FILENO );
		close ( fd [ 0 ] );
		close ( fd [ 1 ] );

		if ( outputFile == "" )
			( void ) execlp ( "dot", "dot", "-Tx11", nullptr );
		else
			( void ) execlp ( "dot", "dot", ( "-T" + outputType ).c_str ( ), "-o", outputFile.c_str ( ), nullptr );
	} else {
		close ( fd [ 0 ] );
		if ( write ( fd [ 1 ], data.c_str ( ), data.size ( ) + 1 ) != ( ssize_t ) data.size ( ) )
			throw exception::CommonException ( "Dot: Failure while passing data to dot." );

		close ( fd [ 1 ] );
	}
}

void Dot::dot ( const std::string & data ) {
	run ( data );
}

void Dot::dot ( const std::string & data, const std::string & outputType_, const std::string & outputFile ) {
	std::string outputType;
	std::transform ( outputType_.begin( ), outputType_.end(), outputType.begin ( ), ::tolower );

	if ( std::find ( allowedTypes.begin( ), allowedTypes.end ( ), outputType ) == allowedTypes.end ( ) ) {
		std::string allowedTypesHelp = std::accumulate ( allowedTypes.begin ( ), allowedTypes.end ( ), std::string ( ), [ ] ( const std::string & res, const std::string & e ) { return res == "" ? e : res + ", " + e; } );
		throw exception::CommonException ( "Dot: Output type is invalid. Allowed types are: " + allowedTypesHelp );
	}

	run ( data, outputType, outputFile );
}

auto DotTx11 = registration::AbstractRegister < Dot, void, const std::string & > ( Dot::dot, abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT, "dot" ).setDocumentation (
"Cli builtin command for DOT format visualization\n\
\n\
@param dot a string containing dot data" );

auto DotFile = registration::AbstractRegister < Dot, void, const std::string &, const std::string &, const std::string & > ( Dot::dot, abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT, "data", "outputType", "outputFile" ).setDocumentation (
"Cli builtin command for DOT format visualization\n\
\n\
@param data a string containing dot data\n\
@param outputType the type of dot created image\n\
@param outputFile the destination file name" );

} /* namespace builtin */

} /* namespace cli */

