/*
 * ExceptionHandler.h
 *
 * Created on: Mar 7, 2019
 * Author: Jan Travnicek
 */

#ifndef EXCEPTION_HANDLER_H_
#define EXCEPTION_HANDLER_H_

#include <exception>
#include <iostream>
#include <functional>

namespace alib {

class ExceptionHandler {
	static int handlerRec ( std::ostream & out, unsigned level, std::vector < std::function < int ( std::ostream &, unsigned ) > >::const_reverse_iterator iter );

	static std::vector < std::function < int ( std::ostream &, unsigned ) > > & handlers ( );

	template < class ExceptionType >
	static void addHandlerDeduce ( std::function < void ( std::ostream &, const ExceptionType & ) > handler, int result ) {
		handlers ( ).push_back ( [ = ] ( std::ostream & out, unsigned level ){
			try {
				throw;
			} catch ( const ExceptionType & exception ) {
				out << level << ' ';
				handler ( out, exception );
				return ExceptionHandler::rethrow_if_nested ( out, exception, level, result );
			}
		} );
	}

	static int rethrow_if_nested ( std::ostream &, const std::exception & e, unsigned level, int result );

public:
	static int handle ( std::ostream & output );

	template < int result, class Callable >
	static void addHandler ( Callable handler ) {
		addHandlerDeduce ( std::function ( handler ), result );
	}
};

} /* namespace alib */

#endif /* EXCEPTION_HANDLER_H_ */
