#!/usr/bin/env bash

function help {
	echo    "$0 MODE OUTPUT" >& 2
	echo -e "  - MODE:   'release' or 'nightly'" >& 2
	echo -e "  - OUTPUT: output directory" >& 2
}

function nightly_pkgver {
	git describe --long --tags --match="v*" |  sed 's/^v//;s/\([^-]*-g\)/r\1/;s/-/./g'
}

function release_pkgver {
	MAJOR=$(sed -En 's/^major = (.*)$/\1/p' CMake/generate.conf)
	MINOR=$(sed -En 's/^minor = (.*)$/\1/p' CMake/generate.conf)
	PATCH=$(sed -En 's/^patch = (.*)$/\1/p' CMake/generate.conf)

	echo "${MAJOR}.${MINOR}.${PATCH}"
}

if [[ $# -ne 2 ]]; then
	help
	exit 1
fi

MODE=$1
OUTPUT=$(realpath $2)

if [[ "$MODE" != "release" && "$MODE" != "nightly" ]]; then
	echo "Mode is not release or nightly" >& 2
	help
	exit 1
fi

# go to source root
cd $(dirname $0)
cd ../../

# create sources tarball
pushd ..
tar -c \
	--exclude=.git -vz \
	--transform 's/automata-library/algorithms-library/' \
	-f $OUTPUT/algorithms-library.tar.gz automata-library
popd

# create distrofiles for obs
if [[ $MODE == "nightly" ]]; then
	sed "s/@PKGVER@/$(nightly_pkgver)/;s/@PKGMODE@/-git/" extra/obs/PKGBUILD > ${OUTPUT}/PKGBUILD
else
	sed "s/@PKGVER@/$(release_pkgver)/;s/@PKGMODE@//" extra/obs/PKGBUILD > ${OUTPUT}/PKGBUILD
fi

# copy patches
cp extra/obs/*.patch ${OUTPUT}/
